/*
 *************************************************************************
 *
 * serdisp_specific_pcd8544.h
 * routines for controlling pcd8544-based displays (found in some nokia phones)
 *
 *************************************************************************
 *
 * copyright (C) 2003-2005  wolfgang astleitner
 * email     mrwastl@users.sourceforge.net
 *
 *************************************************************************
 * This program is free software; you can redistribute it and/or modify   
 * it under the terms of the GNU General Public License as published by   
 * the Free Software Foundation; either version 2 of the License, or (at  
 * your option) any later version.                                        
 *                                                                        
 * This program is distributed in the hope that it will be useful, but    
 * WITHOUT ANY WARRANTY; without even the implied warranty of             
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      
 * General Public License for more details.                               
 *                                                                        
 * You should have received a copy of the GNU General Public License      
 * along with this program; if not, write to the Free Software            
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA              
 * 02111-1307, USA.  Or, point your browser to                            
 * http://www.gnu.org/copyleft/gpl.html                                   
 *************************************************************************
 */

/*
 * Pin-Outs and descriptions for LPH7366
 * 
 * Resolution: 84x48
 * Controller: PCD8544
 *
 * The wiring when connecting to the parallel printer port of a IBM-comp. PC:
 * (I've chosen a wiring similar to the wiring I used for optrex 323)
 *
 *
 *                     +--------------------------+
 *                     |     1 2 3 4 5 6 7 8 9    |
 *                     |     # # # # # # # # #    |             1 .. VDD
 *                     |  ===#=#=#=#=#=#=#=#=#==  |             2 .. SCLK
 *                     +--======================--+             3 .. SI
 *                     |                          |             4 .. D/C
 *                     |                          |             5 .. /CS
 *                     |        rear view         |             6 .. Osc
 *                     |   connector is visible   |             7 .. GND
 *                     |                          |             8 .. Vout
 *                     |         LPH7366          |             9 .. /RES
 *                     |                          |
 *                     +--------------------------+
 *
 *
 *  Signal name  PC            LCD     Signal name
 *  ---------------------------------------------------------
 *  Data 0       2              3      SI   serial data input of LCD
 *  Data 1       3              2      SCLK serial clock line of LCD
 *  Data 2       4              4      D/C  (or sometimes labelled A0) command/data switch
 *  Data 5       7              9      /RES active low reset
 *  Data 7       9              -           Backlight (optional, not on display)
 *  Ground      18              7      GND  Ground for printer port and VDD
 *              --              1      VDD  +5V (up to 7.4 mA) Chip power supply
 *              --              5      /CS  active low chip select (connected to GND)
 *              --              6      Osc  external clock (connected to VDD)
 *              --              8      Vout output of display-internal dc/dc converter (not connected)
 *
 *
 *
 * Pin-Outs and descriptions for LPH7779 (should work for LPH7677, too, have a look at: http://sandiding.tripod.com/lcd.html)
 * 
 * Resolution: 84x48
 * Controller: PCD8544
 *
 * The wiring is nearly the same to LPH7366, with the exception that the LPH7779 has one pin less (no option for connecting an external clock)
 * 
 *
 *                     +-------------------------+
 *                     |     1 2 3 4 5 6 7 8     |
 *                     |     # # # # # # # #     |
 *                     |  ===#=#=#=#=#=#=#=#===  |             1 .. VDD
 *                     +--=====================--+             2 .. SCLK
 *                     |                         |             3 .. SI
 *                     |                         |             4 .. D/C
 *                     |        rear view        |             5 .. /CS
 *                     |  connector is visible   |             6 .. GND
 *                     |                         |             7 .. Vout
 *                     |         LPH7779         |             8 .. /RES
 *                     |                         |
 *                     +-------------------------+
 *
 *
 *  Signal name  PC            LCD     Signal name
 *  ---------------------------------------------------------
 *  Data 0       2              3      SI   serial data input of LCD
 *  Data 1       3              2      SCLK serial clock line of LCD
 *  Data 2       4              4      D/C  (or sometimes named A0) command/data switch
 *  Data 5       7              8      /RES active low reset
 *  Data 7       9              -           Backlight (optional, not on display)
 *  Ground      18              6      GND  Ground for printer port and VDD
 *              --              1      VDD  +5V (? mA) Chip power supply
 *              --              5      /CS   active low chip select (connected to GND)
 *              --              7      Vout output of display-internal dc/dc converter (not connected)
 *
 *
 * Source: http://sandiding.tripod.com/lcd.html             (Berty's page with pin-out and circuits) 
 *         http://sandiding.tripod.com/softan/pcd8544.pdf   (data sheet of PCD8544)
 *
 * Enhancement:   /CS may be connected to GND because it is never used in pcd8544.c 
 *                (it was in one of the first versions)
 */


#ifndef SERDISP_SPECIFIC_PCD8544_H
#define SERDISP_SPECIFIC_PCD8544_H

#include "serdisp_control.h"

#ifdef __cplusplus
extern "C" {
#endif
  serdisp_t* serdisp_pcd8544_setup      (const char* dispname, const char* paramstring);
  void       serdisp_pcd8544_init       (serdisp_t* dd);
  void       serdisp_pcd8544_update     (serdisp_t* dd);
  int        serdisp_pcd8544_setoption  (serdisp_t* dd, const char* option, long value);
  void       serdisp_pcd8544_close      (serdisp_t* dd);
#ifdef __cplusplus
    }
#endif


#endif /* SERDISP_SPECIFIC_PCD8544_H */

