/*
 *************************************************************************
 *
 * serdisp_specific_sed153x.h
 * routines for controlling sed153x-based displays 
 * (eg: optrex323, Alps LSU7S1011A a.k.a. 'pollin'-display)
 *
 *************************************************************************
 *
 * copyright (C) 2003-2005  wolfgang astleitner
 * email     mrwastl@users.sourceforge.net
 *
 *************************************************************************
 * This program is free software; you can redistribute it and/or modify   
 * it under the terms of the GNU General Public License as published by   
 * the Free Software Foundation; either version 2 of the License, or (at  
 * your option) any later version.                                        
 *                                                                        
 * This program is distributed in the hope that it will be useful, but    
 * WITHOUT ANY WARRANTY; without even the implied warranty of             
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      
 * General Public License for more details.                               
 *                                                                        
 * You should have received a copy of the GNU General Public License      
 * along with this program; if not, write to the Free Software            
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA              
 * 02111-1307, USA.  Or, point your browser to                            
 * http://www.gnu.org/copyleft/gpl.html                                   
 *************************************************************************
 */

/*
 * Pin-Outs and descriptions for OPTREX 323
 * 
 * Resolution: 117x62  (sick addressing of pixels)
 * Controller: SED1531
 *
 * The wiring when connecting to the parallel printer port of a IBM-comp. PC:
 * (reference wiring by optrex)
 *
 *  Signal name  PC            LCD     Signal name
 *  ---------------------------------------------------------
 *  Data 0       2             12      SI   serial data input of LCD
 *  Data 1       3             11      SCLK serial Clock line of LCD
 *  Data 2       4              9      D/C  (or sometimes labelled A0) command/data switch
 *  Data 3       5              8      CS   first high active Chip select input
 *  Data 4       6              7      /CS  second low active Chip select input
 *  Data 5       7              6      /RES low active Reset input
 *  Data 7       9              3           high active Backlight input
 *  Ground      18              4      Ground for printer port and +9V
 *  Ground      18             13      Ground for printer port and +5V
 *              --             10      +5V (0.4 mA) Chip power supply
 *              --              2      +9V (125 mA) Backlight power supply
 *
 *
 * Source: http://home.foni.net/~tobias-schirmer/d_lcd.htm
 *         http://home.foni.net/~tobias-schirmer/files/KurzInfo.pdf
 */


/*
 * Pin-Outs and descriptions for ALPS LSU7S1011A
 * 
 * Resolution: 96x32
 * Controller: SED1530
 *
 * The controller may be controlled either using serial or parallel data transer, the display supports both accessing methods.
 *
 * pollin.de sells this display and also a display module kit which accesses the display using serial data transfer.
 *
 *
 *  Signal name  PC           LCD      Signal name
 *  ---------------------------------------------------------
 *  Data 0       2             25     SDATA  (SI) serial data input of LCD
 *  Data 1       3             24      SCLK  serial clock line of LCD
 *  Data 2       4             14        A0  (D/C) command/data switch
 *  Data 3       5             12        CS  chip select 1 (high active)
 *  Data 4       6             11       /CS  chip select 2 (low active)
 *  Data 5       7              9      /RES  low active Reset input
 *  Ground      18              26      GND  Ground for printer port
 *
 *
 * Info:  http://serdisplib.sourceforge.net/ser/sed1530_alps.html   (Information, PDFs, ... about ALPS LSU7S1011A display) 
 */


#ifndef SERDISP_SPECIFIC_SED153X_H
#define SERDISP_SPECIFIC_SED153X_H

#include "serdisp_control.h"

#ifdef __cplusplus
extern "C" {
#endif
  serdisp_t* serdisp_sed153x_setup       (const char* dispname, const char* paramstring);
  void       serdisp_sed153x_init        (serdisp_t* dd);
  void       serdisp_sed153x_update      (serdisp_t* dd);
  int        serdisp_sed153x_setoption   (serdisp_t* dd, const char* option, long value);
  void       serdisp_sed153x_close       (serdisp_t* dd);
#ifdef __cplusplus
    }
#endif


#endif /* SERDISP_SPECIFIC_SED1530_X */

