/*
 *************************************************************************
 *
 * serdisp_specific_sed156x.h
 * routines for controlling sed156x-based displays (eg: nokia 7110 display)
 *
 *************************************************************************
 *
 * copyright (C) 2003-2005  wolfgang astleitner
 * email     mrwastl@users.sourceforge.net
 *
 *************************************************************************
 * This program is free software; you can redistribute it and/or modify   
 * it under the terms of the GNU General Public License as published by   
 * the Free Software Foundation; either version 2 of the License, or (at  
 * your option) any later version.                                        
 *                                                                        
 * This program is distributed in the hope that it will be useful, but    
 * WITHOUT ANY WARRANTY; without even the implied warranty of             
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      
 * General Public License for more details.                               
 *                                                                        
 * You should have received a copy of the GNU General Public License      
 * along with this program; if not, write to the Free Software            
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA              
 * 02111-1307, USA.  Or, point your browser to                            
 * http://www.gnu.org/copyleft/gpl.html                                   
 *************************************************************************
 */

/*
 * Pin-Outs and descriptions for Nokia 7110 display
 * 
 * Resolution: 96x65
 * Controller: SED1565
 *
 * The wiring when connecting to the parallel printer port of a IBM-comp. PC:
 * (I've chosen the same wiring as I used for the pcd8544-based displays.
 *  exception: because /CS is low-active, I used D4 (corresponding to optrex323 wiring) and not D3
 *
 *
 *                         +------------------+
 *                        |    7           1   |
 *                       |     # # # # # # #    |
 *                      |    # # # # # # #       |
 *                     |     8           14       |
 *                     +--======================--+
 *                     |   ==    ==    ==    ==   |   <-- Background LEDs
 *                     |      ##############      |   <-- on-glass controller
 *                     +--======================--+
 *                     |                          |
 *                     |                          |
 *                     |        rear view         |
 *                     |   connector is visible   |
 *                     |                          |
 *                     |        Nokia 7110        |
 *                     |                          |
 *                     +--------------------------+
 *
 *
 *  Signal name  PC            LCD     Signal name
 *  ---------------------------------------------------------
 *  Data 0       2             13      SDATA (SI) serial data input of LCD
 *  Data 1       3              5      SCLK  serial clock line of LCD
 *  Data 2       4              4      D/_C  command/data switch
 *  Data 4       6              2      /CS   chip select (low active)
 *  Data 5       7             14      /RES  low active Reset input
 *  Data 7       9           8, 9            Backlight (switched using a transitor)
 *  Ground      18          10,11      GND   Ground for printer port and +3V
 *              --             12      VDD   +V (Chip power supply, some value around 3V)
 *
 *
 *
 * Source: http://sandiding.tripod.com/lcd.html             (Berty's page with pin-out and circuits) 
 *         http://sandiding.tripod.com/nokialcd7110.html    (nokia 7110 display pinout)
 */



/*
 * Pin-Outs and descriptions for pager NEC 21A (a.k.a 'Skyper') 
 * 
 * Resolution: 96x65
 * Controller: SED1561
 *
 * The wiring when connecting to the parallel printer port of a IBM-comp. PC:
 * (I've chosen the same wiring as I used for the pcd8544-based displays.
 *  exception: because /CS is low-active, I used D4 (corresponding to optrex323 wiring) and not D3
 *
 *
 *
 *                 display board, connector on the right side, controller in the middle of the board
 *
 *
 *
 *                                 top view
 *                                                        
 *                                                        ____ 
 *                                                       |    \
 *                                                  VDD  | =   |
 *                                                       |   = |  Sensor
 *                                                 (nc)  | =   |
 *                                                       |   = |  ?
 *                                                 (nc)  | =   |
 *                                                       |   = |  /WR
 *                                                  VSS  | =   |
 *                                                       |   = |  /RD
 *                                                 /RES  | =   |
 *                                                       |   = |  D7
 *                                                  /CS  | =   |
 *                                                       |   = |  D6
 *          --------------                           A0  | =   |
 *         |              |                              |   = |  D5
 *         |              |                           ?  | =   |
 *         |  LCD-        |                              |   = |  D4
 *         |  Controller  |                           ?  | =   |
 *         |              |                              |   = |  D3
 *         |  SED 1561    |                        BUT4  | =   |
 *         |              |                              |   = |  D2
 *         |              |                        BUT3  | =   |
 *         |              |                              |   = |  D1
 *         |              |                        BUT2  | =   |
 *         |              |                              |   = |  D0
 *         |              |                        BUT1  | =   |
 *         |              |                              |____/   
 *         |              |                              
 *          --------------       
 *                    
 *                    
 *                    
 *  Remarks:                  
 *   Display board is hard-wiring to 8080-mode and parallel interface
 *                    
 *                                       
 *
 *
 *  LCD        Description
 *  ---------------------------------
 *  D0 - D7    Data0 - Data 7
 *  /WR        Write enable input (active-low)
 *  /RD        Read enable input (active-low)
 *  /RES       Reset (active-low)
 *  /CS        Chip select input (active-low)
 *  A0         Command (LOW) or display data (HIGH)
 *  VDD        3.5V (Chip power supply)
 *  VSS        0V
 *
 *
 *
 * Source: measured (i seem to be the first who tries to control this display ...)
 *         
 * Special thanks to TEN, who donated a NEC 21A pager
 */



#ifndef SERDISP_SPECIFIC_SED156X_H
#define SERDISP_SPECIFIC_SED156X_H

#include "serdisp_control.h"

#ifdef __cplusplus
extern "C" {
#endif
  serdisp_t* serdisp_sed156x_setup       (const char* dispname, const char* paramstring);
  void       serdisp_sed156x_init        (serdisp_t* dd);
  void       serdisp_sed156x_update      (serdisp_t* dd);
  int        serdisp_sed156x_setoption   (serdisp_t* dd, const char* option, long value);
  void       serdisp_sed156x_close       (serdisp_t* dd);
#ifdef __cplusplus
    }
#endif


#endif /* SERDISP_SPECIFIC_SED156X_H */

