/*
 *************************************************************************
 *
 * serdisp_specific_i2c.h
 * routines for controlling i2c-based displays (eg: found in some older ericsson)
 *
 *************************************************************************
 *
 * copyright (C) 2003-2005  wolfgang astleitner
 * email     mrwastl@users.sourceforge.net
 *
 *************************************************************************
 * This program is free software; you can redistribute it and/or modify   
 * it under the terms of the GNU General Public License as published by   
 * the Free Software Foundation; either version 2 of the License, or (at  
 * your option) any later version.                                        
 *                                                                        
 * This program is distributed in the hope that it will be useful, but    
 * WITHOUT ANY WARRANTY; without even the implied warranty of             
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      
 * General Public License for more details.                               
 *                                                                        
 * You should have received a copy of the GNU General Public License      
 * along with this program; if not, write to the Free Software            
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA              
 * 02111-1307, USA.  Or, point your browser to                            
 * http://www.gnu.org/copyleft/gpl.html                                   
 *************************************************************************
 */

/*
 * Pin-Outs and descriptions for Ericsson T28i display
 * 
 * Resolution: 101x33
 * Controller: n/a (berty's page: cheap version of PCF8558)
 *
 * The wiring when connecting to the parallel printer port of a IBM-comp. PC:
 * (I've chosen the same wiring as I used for the pcd8544-based displays,
 *  but only D0, D1, and D7 (=SDA / SI, SCLK, and BG-light) are used
 *
 *
 *                     +----------------------------+
 *                     |         1 2 3 4 5          |
 *                     |         # # # # #          |
 *                     +------================------+
 *                     |                            |
 *                     |          rear view         |
 *                     |    connector is visible    |
 *                     |                            |
 *                     |           ??n/a??          |
 *                     |                            |
 *                     +----------------------------+
*
 *
 *  Signal name  PC            LCD     Signal name
 *  ----------------------------------------------------------
 *  Data 0       2              3      SDATA  i2c-SDA
 *  Data 1       3              5      SCLK   i2c-SCLK
 *  Ground      18              4      GND    Ground
 *              --              1      VLCD   +6.4V (lcd power supply, some value around 5-6V)
 *              --              2      VLOGIC +2.7V (logic power supply, some value around 2.7-5V)
 *
 *
 *
 * Source: http://rifer.narod.ru/i2cldc.html          (Rifers' I2C-pages (russian))
 *         http://sandiding.tripod.com/lcdt28.html    (Berty's page / Ericsson T20 / T28 / T29 LCD Pins) 
 */



#ifndef SERDISP_SPECIFIC_I2C_H
#define SERDISP_SPECIFIC_I2C_H

#include "serdisp_control.h"


#ifdef __cplusplus
extern "C" {
#endif
  serdisp_t* serdisp_i2c_setup      (const char* dispname, const char* paramstring);
  void       serdisp_i2c_init       (serdisp_t* dd);
  void       serdisp_i2c_update     (serdisp_t* dd);
  int        serdisp_i2c_setoption  (serdisp_t* dd, const char* option, long value);
  void       serdisp_i2c_close      (serdisp_t* dd);
#ifdef __cplusplus
    }
#endif


#endif /* SERDISP_SPECIFIC_I2C_H */

