/*
 *************************************************************************
 *
 * serdisp_specific_nokcol.h
 * routines for controlling nokia colour displays using the following controllers:
 * * Seiko S1D15F14
 *
 *************************************************************************
 *
 * copyright (C) 2003-2005  wolfgang astleitner
 * email     mrwastl@users.sourceforge.net
 *
 *************************************************************************
 * This program is free software; you can redistribute it and/or modify   
 * it under the terms of the GNU General Public License as published by   
 * the Free Software Foundation; either version 2 of the License, or (at  
 * your option) any later version.                                        
 *                                                                        
 * This program is distributed in the hope that it will be useful, but    
 * WITHOUT ANY WARRANTY; without even the implied warranty of             
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      
 * General Public License for more details.                               
 *                                                                        
 * You should have received a copy of the GNU General Public License      
 * along with this program; if not, write to the Free Software            
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA              
 * 02111-1307, USA.  Or, point your browser to                            
 * http://www.gnu.org/copyleft/gpl.html                                   
 *************************************************************************
 */

/*
 * Pin-Out and descriptions for Nokia 3510i displays
 * 
 * Resolution: 98x67, 4096 colours
 * Controller: Epson S1D15G14 Series
 *
 * The wiring when connecting to the parallel printer port of a IBM-comp. PC:
 * (I've chosen a wiring similar to wirings used in former projects)
 *
 *
 *                     +------------------------+
 *                     |     1 2 3 4 5 6 7 8    |
 *                     |     # # # # # # # #    |             1 .. /RES
 *                     |  ===#=#=#=#=#=#=#=#==  |             2 .. /CS
 *                     +--====================--+             3 .. GND
 *                     |                        |             4 .. SI (SDATA)
 *                     |                        |             5 .. SCLK
 *                     |        rear view       |             6 .. Vio
 *                     |  connector is visible  |             7 .. Vflash
 *                     |                        |             8 .. Vout
 *                     |         N3510i         |             
 *                     |                        |
 *                     +------------------------+
 *
 *
 *  Signal name  PC            LCD     Signal name
 *  ---------------------------------------------------------
 *  Data 0       2              4      SI     serial data input of LCD
 *  Data 1       3              5      SCLK   serial clock line of LCD
 *  Data 2       4              2      /CS    chip select
 *  Data 5       7              1      /RES   active low reset
 *  Data 7       9              -             Backlight (optional, not on display)
 *  Ground      18              3      GND    Ground for printer port and Vdigital and Vbooster
 *              --              6      Vio    1.8V chip power
 *              --              7      Vflash 2.8V display power
 *              --              8      Vout   output of display-internal dc/dc converter 
 *                                            (Vout -> capacitor -> GND   to avoid noise)
 *
 *
 */

#ifndef SERDISP_SPECIFIC_NOKCOL_H
#define SERDISP_SPECIFIC_NOKCOL_H

#include "serdisp_control.h"


#ifdef __cplusplus
extern "C" {
#endif
  serdisp_t* serdisp_nokcol_setup       (const char* dispname, const char* paramstring);
  void       serdisp_nokcol_init        (serdisp_t* dd);
  void       serdisp_nokcol_update      (serdisp_t* dd);
  int        serdisp_nokcol_setoption   (serdisp_t* dd, const char* option, long value);
  void       serdisp_nokcol_close       (serdisp_t* dd);
#ifdef __cplusplus
    }
#endif


#endif /* SERDISP_SPECIFIC_NOKCOL_H */

